MODULE Audio1;

(* Ausgabe einer Tonfolge ueber einen Stereokanal *)

FROM Audio IMPORT
	allocate, allocFailed, audioName, free, IOAudio,
	(* in C: ADCMD_ALLOCATE, IOERR_ALLOCFAILED usw *)
	IOAudioPtr, noWait, pervol;
FROM ExecD IMPORT
	IOFlagSet, MemReqs, MemReqSet, MsgPortPtr, openFail,
	(* IOFlagSet enthaelt in C-Notation: ADIOF_xxx *)
	quick, start, stop, write;
	(* start in C: CMD_START *)
FROM ExecL IMPORT
	AllocMem, CloseDevice, FreeMem, GetMsg, OpenDevice,
	Wait, WaitPort;
FROM ExecSupport IMPORT
	BeginIO, CreatePort, DeletePort;
FROM SYSTEM IMPORT
	ADDRESS, ADR, LONGSET, TSIZE;

TYPE
     Welle    = ARRAY[0..15] OF SHORTINT;(* Sinusschwing. *)
     WellePtr = POINTER TO Welle;        (* im CHIP-Ram   *)

     Noten    = ARRAY[0..29] OF INTEGER; (* fuer Tonfolge *)
     NotenPtr = POINTER TO INTEGER;

VAR
     wellePtr : WellePtr; (* fuer Zugriff auf Sinuswelle im
                             CHIP-RAM *)

     audioOpen     : BOOLEAN;          (* Device geffnet *)
     deviceIOBlock,            (* Open & Close Audio Dev. *)

     stereoIOBlock : IOAudioPtr; (* Anfordern u. Aufgeben
                                    des (doppelten) Stereo-
                                    Kanals  sowie f. Komman-
                                    dos an beide Kanle
                                    gleichzeitig *)

     (* fr jeden Kanal den Prototyp eines initialisierten
        IOBlocks, Zeiger darauf
      *)
     liProto,     reProto     : IOAudioPtr;

     (* 2 * RAM fuer Tonfolge
     *)
     soundLinks,  soundRechts : Noten;
     notLiPtr,    notRePtr    : NotenPtr;(* Zeiger darauf *)


PROCEDURE NotenLaden;

(* eine Oktave im Zweiklang *)

VAR noPtr : NotenPtr;

BEGIN
  noPtr  := ADR(soundLinks);
  noPtr^ := 1695;INC(noPtr, 2);noPtr^ := 1510;INC(noPtr, 2);
  noPtr^ := 1345;INC(noPtr, 2);noPtr^ := 1270;INC(noPtr, 2);
  noPtr^ := 1131;INC(noPtr, 2);noPtr^ := 1008;INC(noPtr, 2);
  noPtr^ :=  898;INC(noPtr, 2);noPtr^ :=  847;

  noPtr  := ADR(soundRechts);
  noPtr^ := 1270;INC(noPtr, 2);noPtr^ := 1131;INC(noPtr, 2);
  noPtr^ := 1008;INC(noPtr, 2);noPtr^ :=  898;INC(noPtr, 2);
  noPtr^ :=  847;INC(noPtr, 2);noPtr^ :=  755;INC(noPtr, 2);
  noPtr^ :=  673;INC(noPtr, 2);noPtr^ :=  635;
END NotenLaden;


PROCEDURE WelleInsChip(sinusPtr : WellePtr);

(* Samples zur Beschreibung der Wellenform ins CHIP-RAM
   kopieren.
*)
VAR index : SHORTINT;

BEGIN
                                          (* 1. Halbwelle *)
  sinusPtr^[0] :=   0; sinusPtr^[1] :=  75;
  sinusPtr^[2] := 103; sinusPtr^[3] := 121;
  sinusPtr^[4] := 127; sinusPtr^[5] := 121;
  sinusPtr^[6] := 103; sinusPtr^[7] :=  75;
                                          (* 2. Halbwelle *)
  FOR index := 8 TO 15 DO
    sinusPtr^[index] := - sinusPtr^[index-8];
  END;
END WelleInsChip;


PROCEDURE OeffneAudioDevice (VAR audioDev : IOAudioPtr)
                                                   :BOOLEAN;

(* Legt eine IOAudio-Struktur an u. ffnet das AudioDevice
   ohne Kanle anzufordern. CloseDevice ist mit einem Zeiger
   auf diese Struktur aufzurufen.
*)
BEGIN
  audioDev := AllocMem(SIZE(IOAudio),
                       MemReqSet{public, memClear});
  IF audioDev # NIL THEN          (* IO-Struktur angelegt *)
    OpenDevice(ADR(audioName), 0, audioDev, LONGSET{});
    RETURN audioDev^.request.error # openFail;
  ELSE
    RETURN FALSE
  END;
END OeffneAudioDevice;


PROCEDURE SchliesseAudioDevice(VAR audioDev : IOAudioPtr);

BEGIN
  IF audioDev # NIL THEN
    CloseDevice(audioDev);
    FreeMem(audioDev, SIZE(IOAudio));
    audioDev := NIL;
  END;
END SchliesseAudioDevice;


PROCEDURE SynchroIO(VAR ioBlock:IOAudioPtr;
                       kommando:CARDINAL;
                       ioFlags : IOFlagSet);

(* Nachbildung von DoIO mit der vorteilhaften Ergaenzung,
   IO-Flags wahlweise zu setzen.
*)
VAR dummy : ADDRESS;

BEGIN
  WITH ioBlock^.request DO
    command := kommando;

    (* quickI/O Flag setzen *)
    flags := ioFlags + quick;

    BeginIO(ioBlock);

    (* Falls quick I/O Flag nicht mehr gesetzt ist, dann
       auf Antwort am MsgPort warten, sonst ist I/O Request
       schon abgearbeitet
    *)
    IF NOT (flags * quick = quick) THEN
      (* WaitIO(ioBlock);inkorrekt mit OS 1.2/1.3 deshalb *)
      WITH ioBlock^.request.message DO
        WaitPort(replyPort);
        dummy := GetMsg(replyPort);
      END;
    END;
  END;
END SynchroIO;


PROCEDURE StereoKanalAnfordern(audioDev : ADDRESS)
                                        : IOAudioPtr;

(* beliebigen, doppelt ausgelegten Kanal anfordern und als
   Ergebnis die Adresse einer angelegten und vollstaendig
   initialisierten (einschl. Msg Port) IOAudio-Struktur
   uebergeben oder NIL (bei Fehlschlag)
*)
CONST AllocationArray = 03050A0CH;
(*                       | | | |        jedes der vier Bytes
            Kanaele: 0 + 1 | | |        repraesentiert eine
                       0 + 2 | |        moegliche Kanalkom-
                         1 + 3 |        bination
                           2 + 3
*)
VAR   stereoKanal        : IOAudioPtr;
      kanalKombinationen : LONGCARD;

BEGIN
  stereoKanal := NIL;
  kanalKombinationen := AllocationArray;
  IF audioDev # NIL THEN                (* zur Sicherheit *)
    stereoKanal := AllocMem(SIZE(IOAudio),
                            MemReqSet{public, memClear});
    (* somit ist auch allocKey mit null initialisiert *)

    IF stereoKanal # NIL THEN (* IO-Struktur angelegt *)
      WITH stereoKanal^ DO

        (* Port fuer Messages Device an Audio1 (Task) *)
        request.message.replyPort := CreatePort(NIL, 0);

        IF request.message.replyPort # NIL THEN
                                    (* haben wir den Port *)
          request.message.node.pri := 10;
          request.device := audioDev;
          data := ADR(kanalKombinationen);
          length := 4;        (* Anzahl der Kombinationen *)

          (* sofort ber Kanle verfgen und keinesfalls
             warten, bis zugriffsbereit;
             ggf. allocFailed melden *)

          SynchroIO(stereoKanal, allocate, noWait);
          (* DoIO waere nicht moeglich, weil I/O Flag
             noWait zu benutzen ist.
          *)
          IF stereoKanal^.request.error # 0 THEN
            DeletePort(request.message.replyPort);
            FreeMem(stereoKanal, SIZE(IOAudio));
            stereoKanal := NIL;
          END;
        ELSE
          FreeMem(stereoKanal, SIZE(IOAudio));
          stereoKanal := NIL;
        END; (* IF request.message ... *)
      END; (* WITH *)
    END; (* IF stereoKanal *)
  END; (* IF audioDev *)
  RETURN stereoKanal;
END StereoKanalAnfordern;


PROCEDURE StereoKanalFreigeben(VAR stereoKanal: IOAudioPtr);

BEGIN
  IF stereoKanal # NIL THEN
    WITH stereoKanal^ DO
      SynchroIO(stereoKanal, free, IOFlagSet{});
      (* DoIO ware moeglich, da free synchron arbeitet *)

      IF request.error = 0 THEN
        DeletePort(request.message.replyPort);
        FreeMem(stereoKanal, SIZE(IOAudio));
        stereoKanal := NIL;
      END;
    END;
  END;
END StereoKanalFreigeben;


(* Der Stereokanal ist an einenI/O Request gebunden. Beide
   Kanaele sind jedoch getrennt mit Daten zu versorgen und
   benoetigen gesonderte I/O Requests
*)
PROCEDURE StereoKanalAufsplitten(stereo : IOAudioPtr;
                             VAR links,
                                 rechts : IOAudioPtr);
VAR kanal : POINTER TO LONGSET;
BEGIN
  rechts := NIL;                        (* zur Sicherheit *)
  links := AllocMem(SIZE(IOAudio),
                    MemReqSet{public, memClear});

  IF links # NIL THEN
    (* MsgPort, allocKey, device und Units kopieren *)
    links^ := stereo^;

    (* Zeiger auf Unit, um Kanal 0 oder 3 auszublenden
       (AND %1001)
    *)
    kanal := ADR(links^.request.unit);
    kanal^ := kanal^ * LONGSET{3,0};

    (* Port anlegen fuer Antw. li. Kanal an Audio1-Task *)
    WITH links^.request.message DO
      replyPort := CreatePort(NIL, 0);
    END;

    rechts := AllocMem(SIZE(IOAudio),
                       MemReqSet{public, memClear});
    IF rechts # NIL THEN
      (* MsgPort, alocKey und Units kopieren *)
      rechts^ := stereo^;
      kanal := ADR(rechts^.request.unit);

      (* Kanal 1 oder 2 ausblenden (AND %0110) *)
      kanal^ := kanal^ * LONGSET{2,1};

      WITH rechts^.request.message DO
        replyPort := CreatePort(NIL, 0);
      END;

    ELSE  (* kein Port anlegbar *)
      DeletePort(links^.request.message.replyPort);
      FreeMem(links, SIZE(IOAudio));
      links := NIL;
    END;
  END;
END StereoKanalAufsplitten;



PROCEDURE IOBlockAufgeben(VAR ioBlock : IOAudioPtr);

BEGIN
  DeletePort(ioBlock^.request.message.replyPort);
  FreeMem(ioBlock, SIZE(IOAudio));
  ioBlock := NIL;
END IOBlockAufgeben;


PROCEDURE EinzelSteuerungAufgeben(VAR links,
                                      rechts : IOAudioPtr);
BEGIN
  IOBlockAufgeben(links);
  IOBlockAufgeben(rechts);
END EinzelSteuerungAufgeben;


PROCEDURE StereoKanalStoppen(stereo : IOAudioPtr);

BEGIN
  SynchroIO(stereo, stop, IOFlagSet{});
  (* DoIO ware moeglich, da stop synchron arbeitet *)
END StereoKanalStoppen;


PROCEDURE StereoKanalStarten(stereo : IOAudioPtr);

BEGIN
  SynchroIO(stereo, start, IOFlagSet{});
  (* DoIO ware moeglich, da start synchron arbeitet *)
END StereoKanalStarten;


PROCEDURE SoundLaden(VAR kanalIO:IOAudio;
                     VAR notenPtr : NotenPtr);

(* die ersten drei Samples werden mit dieser Routine gela-
   den, um die I/O Requests vollst. zu initialisieren     *)
BEGIN
    WITH kanalIO DO
      request.command := write; (* single channel cmd *)
      request.flags   := pervol;(* volume regeln *)
      data            := wellePtr;
      length          := TSIZE(Welle);
      volume          := 50;    (* Lautstaerke *)

      (* cycles fuer gleichlange Toene berechnen *)

      cycles          := 49200 DIV notenPtr^;(*stnd Dauer *)
      period          := notenPtr^;
    END;
    INC(notenPtr, 2);            (* auf die naechste Note *)
END SoundLaden;


PROCEDURE SoundNachLaden(VAR kanalIO:IOAudio;
                     VAR notenPtr : NotenPtr);

(* fuettert das Audio-Device mit Toenen nach und benutzt
   dazu bereits initialisierte I/O Requests
*)
BEGIN
    WITH kanalIO DO
      cycles          := 49200 DIV notenPtr^;
      period          := notenPtr^;
    END;
    INC(notenPtr, 2);            (* auf die naechste Note *)
END SoundNachLaden;


PROCEDURE SoundSpielen;

CONST Sounds = 6;          (* drei I/O Requests pro Kanal *)
TYPE  SoundRequest = ARRAY [0..Sounds-1] OF IOAudio;

VAR portSignale,
    erhalteneSig    : LONGSET;
    soundReqPtr     : POINTER TO SoundRequest;
    returnedIOBlock : IOAudioPtr;
    unbeantwortIOs  : INTEGER; (* abgesandte, unbeantwortete
                                  I/O Requsts *)
    index           : INTEGER;
    liSigBit,
    reSigBit        : SHORTINT;

BEGIN
  (* Initialisierung *)

  notLiPtr := ADR(soundLinks);        (* fuer Zugriff auf *)
  notRePtr := ADR(soundRechts);       (* Tonfolge *)

  (* Sig Bits zur Vereinfachung kopieren *)

  liSigBit := liProto^.request.message.replyPort^.sigBit;
  reSigBit := reProto^.request.message.replyPort^.sigBit;
  portSignale := LONGSET{liSigBit, reSigBit};

  (* beide Kanaele stoppen, da sie nicht gleichzeitig mit
     Sound-Daten gefuettert werden koennen
  *)
  StereoKanalStoppen(stereoIOBlock);

  (* fuer jeden Kanal 3 I/O Requests en bloc anlegen
  *)
  soundReqPtr := AllocMem(SIZE(SoundRequest),
                          MemReqSet{public, memClear});

  (* 6 I/O Requests mit den Inhalten der Prototypen fuer den
     linken u. den rechten Kanal initialisieren, sie mit
     Sound-Daten laden und in die beiden Warteschlangen der
     zwei Kanaele einfuegen
  *)
  unbeantwortIOs := Sounds;    (* Anzahl der I/O-Requests *)
  FOR index := 0 TO (Sounds - 1) DO
    IF (index MOD 2) = 0 THEN
      soundReqPtr^[index] := liProto^;
      SoundLaden(soundReqPtr^[index], notLiPtr);
    ELSE
      soundReqPtr^[index] := reProto^;
      SoundLaden(soundReqPtr^[index], notRePtr);
    END;
    BeginIO(ADR(soundReqPtr^[index]));
  END;

  (* Tonfolge abspielen
  *)
  StereoKanalStarten(stereoIOBlock); (* beide zugleich *)

  (* warten bis ein I/O Request beantwortet zurueckgeschickt
     ist, diesen mit neuen Sound-Daten versehen, sofern noch
     welche abzuspielen sind (notxxPtr^# NIL)
  *)
  REPEAT
    (* Msg Port beider Kanaele abhoeren
    *)
    erhalteneSig := Wait(portSignale);

    IF liSigBit IN erhalteneSig THEN    (* Msg von links? *)
      returnedIOBlock :=
                 GetMsg(liProto^.request.message.replyPort);
      DEC(unbeantwortIOs);    (* ein unbeantwort. weniger *)
      IF notLiPtr^ # 0 THEN (* noch ungespielte Noten da? *)
        SoundNachLaden(returnedIOBlock^, notLiPtr);
        BeginIO(returnedIOBlock);
        INC(unbeantwortIOs);     (* ein unbeantwort. mehr *)
      END;
    END;

    IF reSigBit IN erhalteneSig THEN   (* Msg von rechts? *)
      returnedIOBlock :=
                 GetMsg(reProto^.request.message.replyPort);
      DEC(unbeantwortIOs);
      IF notRePtr^ # 0 THEN
        SoundNachLaden(returnedIOBlock^, notRePtr);
        BeginIO(returnedIOBlock);
        INC(unbeantwortIOs); (* 1 Block ging auf die Reise*)
      END;
    END;
  UNTIL unbeantwortIOs = 0;(*alle I/O-Requests beantwortet*)

  FreeMem(soundReqPtr, SIZE(SoundRequest));
END SoundSpielen;


BEGIN
  NotenLaden;                   (* abzuspielende Tonfolge *)

  (* CHIP-Ram fuer Sinus-Sample anfordern
  *)
  wellePtr := AllocMem(TSIZE(Welle),
                       MemReqSet{chip, memClear});
  IF wellePtr # NIL THEN
    WelleInsChip(wellePtr); (* Samples ins Chip-Ram laden *)
    audioOpen := OeffneAudioDevice(deviceIOBlock);
    IF audioOpen THEN
      stereoIOBlock :=
        StereoKanalAnfordern(deviceIOBlock^.request.device);
      IF stereoIOBlock # NIL THEN
        StereoKanalAufsplitten(stereoIOBlock,
                               liProto, reProto);
        SoundSpielen;
      END;
    END;

    IF audioOpen THEN
      EinzelSteuerungAufgeben(liProto, reProto);
      StereoKanalFreigeben(stereoIOBlock);
      SchliesseAudioDevice(deviceIOBlock);
    END;
    FreeMem(wellePtr, TSIZE(Welle));
  END;
END Audio1.
